/* 
 *	Copyright (C) 2005 Media Portal
 *	http://mediaportal.sourceforge.net
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

using System;
using System.Windows.Forms;
using MediaPortal.GUI.Library;
using MediaPortal.Util;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace ProcessPlugins.CallerId
{
	/// <summary>
	/// Summary description for OutlookHelper.
	/// </summary>
	public class OutlookHelper
	{
    public struct Caller
    {
      public string Name;
      public string Type;
      public bool HasPicture;
    }

    static public Caller OutlookLookup(string outlookQuery)
    {
      Caller found = new Caller();

      try
      {
        Outlook.Application olApplication = new Outlook.Application();
        Outlook.NameSpace olNamespace = olApplication.GetNamespace("mapi");
        Outlook.MAPIFolder olContacts = olNamespace.GetDefaultFolder(Outlook.OlDefaultFolders.olFolderContacts);
        Outlook.Items olItems = (Outlook.Items)olContacts.Items;

        string sFilter 
          = "[AssistantTelephoneNumber] = '"   + outlookQuery + "' Or "
          + "[Business2TelephoneNumber] = '"   + outlookQuery + "' Or "
          + "[BusinessFaxNumber] = '"          + outlookQuery + "' Or "
          + "[BusinessTelephoneNumber] = '"    + outlookQuery + "' Or "
          + "[CallbackTelephoneNumber] = '"    + outlookQuery + "' Or "
          + "[CarTelephoneNumber] = '"         + outlookQuery + "' Or "
          + "[CompanyMainTelephoneNumber] = '" + outlookQuery + "' Or "
          + "[Home2TelephoneNumber] = '"       + outlookQuery + "' Or "
          + "[HomeFaxNumber] = '"              + outlookQuery + "' Or "
          + "[HomeTelephoneNumber] = '"        + outlookQuery + "' Or "
          + "[ISDNNumber] = '"                 + outlookQuery + "' Or "
          + "[MobileTelephoneNumber] = '"      + outlookQuery + "' Or "
          + "[OtherFaxNumber] = '"             + outlookQuery + "' Or "
          + "[OtherTelephoneNumber] = '"       + outlookQuery + "' Or "
          + "[PrimaryTelephoneNumber] = '"     + outlookQuery + "' Or "
          + "[RadioTelephoneNumber] = '"       + outlookQuery + "' Or "
          + "[TelexNumber] = '"                + outlookQuery + "' Or "
          + "[HomeTelephoneNumber] = '"        + outlookQuery + "' Or "
          + "[TTYTDDTelephoneNumber] = '"      + outlookQuery + "'";

        Outlook.ContactItem olContactItem = (Outlook.ContactItem)olItems.Find(sFilter);

        if (olContactItem != null)
        {
          if      (olContactItem.AssistantTelephoneNumber   == outlookQuery) found.Type = "Assistant Telephone";
          else if (olContactItem.Business2TelephoneNumber   == outlookQuery) found.Type = "Business 2 Telephone";
          else if (olContactItem.BusinessFaxNumber          == outlookQuery) found.Type = "Business Fax";
          else if (olContactItem.BusinessTelephoneNumber    == outlookQuery) found.Type = "Business Telephone";
          else if (olContactItem.CallbackTelephoneNumber    == outlookQuery) found.Type = "Callback Telephone";
          else if (olContactItem.CarTelephoneNumber         == outlookQuery) found.Type = "Car Telephone";
          else if (olContactItem.CompanyMainTelephoneNumber == outlookQuery) found.Type = "Company Main Telephone";
          else if (olContactItem.Home2TelephoneNumber       == outlookQuery) found.Type = "Home 2 Telephone";
          else if (olContactItem.HomeFaxNumber              == outlookQuery) found.Type = "Home Fax";
          else if (olContactItem.HomeTelephoneNumber        == outlookQuery) found.Type = "Home Telephone";
          else if (olContactItem.ISDNNumber                 == outlookQuery) found.Type = "ISDN";
          else if (olContactItem.MobileTelephoneNumber      == outlookQuery) found.Type = "Mobile Telephone";
          else if (olContactItem.OtherFaxNumber             == outlookQuery) found.Type = "Other Fax";
          else if (olContactItem.OtherTelephoneNumber       == outlookQuery) found.Type = "Other Telephone";
          else if (olContactItem.PrimaryTelephoneNumber     == outlookQuery) found.Type = "Primary Telephone";
          else if (olContactItem.RadioTelephoneNumber       == outlookQuery) found.Type = "Radio Telephone";
          else if (olContactItem.TelexNumber                == outlookQuery) found.Type = "Telex";
          else if (olContactItem.TTYTDDTelephoneNumber      == outlookQuery) found.Type = "TTYTDD Telephone";

          found.Name = olContactItem.FullName;

          string applicationPath = Application.ExecutablePath;
          applicationPath = System.IO.Path.GetFullPath(applicationPath);
          applicationPath = System.IO.Path.GetDirectoryName(applicationPath);
          if (olContactItem.Attachments[1] != null)
          {
            olContactItem.Attachments[1].SaveAsFile(applicationPath + @"\" + Thumbs.Yac + @"\" + olContactItem.Attachments[1].FileName);
            found.HasPicture = true;
          }
        }
        olContactItem = null;
        olItems = null;
        olContacts = null;
        olNamespace = null;
        olApplication = null;
      }
      catch (Exception ex)
      {
        Log.Write("Outlook exception: {0}", ex.Message);
      }

      return found;
    }
	}
}