using System;
using System.Collections;
using System.Xml;
using System.Threading;
using MediaPortal.Radio.Database;
using MediaPortal.GUI.Library;
using MediaPortal.TV.Recording;
using MediaPortal.Util;
using MediaPortal.GUI.Settings.Wizard;
using DShowNET;
namespace WindowPlugins.GUISettings.Wizard.Analog
{
	/// <summary>
	/// Summary description for GUIWizardAnalogTuneRadio.
	/// </summary>
	public class GUIWizardAnalogTuneRadio: GUIWindow
	{
		[SkinControlAttribute(26)]			protected GUILabelControl lblChannelsFound=null;
		[SkinControlAttribute(27)]			protected GUILabelControl lblStatus=null;
		[SkinControlAttribute(24)]			protected GUIListControl  listChannelsFound=null;
		[SkinControlAttribute(5)]			  protected GUIButtonControl  btnNext=null;
		[SkinControlAttribute(25)]			protected GUIButtonControl  btnBack=null;
		[SkinControlAttribute(20)]			protected GUIProgressControl progressBar=null;

		int card=0;
		
		int        currentFrequencyIndex=0;
		bool updateList=false;
		int newChannels=0;
		static ArrayList listRadioChannels = new ArrayList();

		const int minFreq=87500000;
		const int maxFreq=108000000;
		const int stepFreq=100000;
		public GUIWizardAnalogTuneRadio()
		{
			GetID=(int)GUIWindow.Window.WINDOW_WIZARD_ANALOG_SCAN_RADIO;
		}
    
		public override bool Init()
		{
			return Load (GUIGraphicsContext.Skin+@"\wizard_tvcard_analog_scanradio.xml");
		}
		protected override void OnPageDestroy(int newWindowId)
		{
			base.OnPageDestroy (newWindowId);
			GUIGraphicsContext.VMR9Allowed=true;
		}

		protected override void OnPageLoad()
		{
			base.OnPageLoad ();
			GUIGraphicsContext.VMR9Allowed=false;
			btnNext.Disabled=true;
			btnBack.Disabled=true;
			progressBar.Percentage=0;
			progressBar.Disabled=false;
			progressBar.IsVisible=true;
			UpdateList();
			Thread WorkerThread = new Thread(new ThreadStart(ScanThread));
			WorkerThread.ApartmentState=ApartmentState.STA;
			//WorkerThread.IsBackground=true;
			WorkerThread.Start();
		}
		public void ScanThread()
		{
			listRadioChannels.Clear();
			newChannels=0;;
			TVCaptureDevice captureCard=null;
			card = Int32.Parse( GUIPropertyManager.GetProperty("#WizardCard"));
			if (card >=0 && card < Recorder.Count)
			{
				captureCard =Recorder.Get(card);
				
			}
			else
			{
				btnNext.Disabled=false;
				btnBack.Disabled=false;
				return;
			}
			try
			{
				updateList=false;
				if (captureCard==null) return;
				currentFrequencyIndex=minFreq;
				while (true)
				{
					if (currentFrequencyIndex >= maxFreq)
					{
						btnNext.Disabled=false;
						btnBack.Disabled=false;
						return;
					}

					UpdateStatus();
					ScanNextFrequency(captureCard,0);
					if (captureCard.SignalPresent())
					{
						ScanChannels(captureCard);
					}
					currentFrequencyIndex+=stepFreq;
				}
			}
			catch(Exception ex)
			{
				Log.WriteFile(Log.LogType.Log,true,"ex:{0} {1} {2}", ex.Message,ex.Source,ex.StackTrace);
			}
			finally
			{
				captureCard.DeleteGraph();
				progressBar.Percentage=100;
				lblChannelsFound.Label=String.Format("Finished, found {0} tv channels",newChannels);
				lblStatus.Label="Press Next to continue the setup";
				GUIControl.FocusControl(GetID,btnNext.GetID);
				captureCard=null;

			}
		}
		void ScanChannels(TVCaptureDevice captureCard)
		{
			Log.Write("Analog-scan:ScanChannels() {0}/{1}",currentFrequencyIndex,200);
			if (currentFrequencyIndex < 0 || currentFrequencyIndex >=200) return;


			System.Threading.Thread.Sleep(400);
			RadioStation chan = new RadioStation();
			chan.Name=String.Format("Channel{0}",currentFrequencyIndex);
			chan.Frequency=currentFrequencyIndex;
			listRadioChannels.Add(chan);
			updateList=true;
			newChannels++;
		}

		void ScanNextFrequency(TVCaptureDevice captureCard,int offset)
		{
			Log.Write("Analog-scan:ScanNextFrequency() {0}/{1}",currentFrequencyIndex,maxFreq);
			if (currentFrequencyIndex <minFreq) currentFrequencyIndex =minFreq;
			if (currentFrequencyIndex >=maxFreq) return;



			System.Threading.Thread.Sleep(400);
			if (!captureCard.SignalPresent())
				System.Threading.Thread.Sleep(400);

			Log.WriteFile(Log.LogType.Capture,"Analog-scan:tune:{0}",currentFrequencyIndex);

			RadioStation station = new RadioStation();
			float freq=(float)currentFrequencyIndex;
			freq/=1000000f;
			station.Name=String.Format("{0} FM",freq.ToString("f2"));
			station.Frequency=currentFrequencyIndex;
			captureCard.StartRadio(station);
			Log.WriteFile(Log.LogType.Capture,"Analog-scan:tuned");
			return;
		}

		public override void Process()
		{
			if (updateList)
			{
				UpdateList();
				updateList=false;
			}
	
			base.Process ();
		}


		void UpdateList()
		{
			Log.Write("UpdateList()");
			listChannelsFound.Clear();
			if (listRadioChannels.Count==0)
			{
				GUIListItem item = new GUIListItem();
				item.Label="No stations found";
				item.IsFolder=false;
				listChannelsFound.Add(item);
				Log.Write("UpdateList() done");
				return;

			}
			int count=1;
			foreach (RadioStation chan in listRadioChannels)
			{
				GUIListItem item = new GUIListItem();
				item.Label=String.Format("{0}. {1}", count,chan.Name);
				item.IsFolder=false;
				string strLogo=Utils.GetCoverArt(Thumbs.TVChannel,chan.Name);
				if (!System.IO.File.Exists(strLogo))
				{
					strLogo="defaultVideoBig.png";
				}
				item.ThumbnailImage=strLogo;
				item.IconImage=strLogo;
				item.IconImageBig=strLogo;
				listChannelsFound.Add(item);
				count++;
			}
			listChannelsFound.ScrollToEnd();
			Log.Write("UpdateList() done");
		}
		void UpdateStatus()
		{
			int currentFreq=currentFrequencyIndex;
			if (currentFrequencyIndex<minFreq) currentFreq=minFreq;
			float percent = ((float)(currentFreq-minFreq)) / ((float)(maxFreq-minFreq));
			percent *= 100.0f;
			
			progressBar.Percentage=(int)percent;
			float freq=(float)currentFreq;
			freq/=1000000f;
			string description=String.Format("{0} FM", freq.ToString("f2"));
			lblChannelsFound.Label=description;
			lblStatus.Label=String.Format("Found {0} radio stations",newChannels);
			Log.Write("Analog-scan:ScanChannels() done");
		}

		protected override void OnClicked(int controlId, GUIControl control, MediaPortal.GUI.Library.Action.ActionType actionType)
		{
			if (control==btnNext)
			{
				GUIWindowManager.ActivateWindow((int)GUIWindow.Window.WINDOW_WIZARD_ANALOG_RENAME_RADIO);
				return;
			}
			base.OnClicked (controlId, control, actionType);
		}
		static public ArrayList RadioStationsFound
		{
			get { return listRadioChannels;}
		}
	}
}
