/* 
 *	Copyright (C) 2005 Media Portal
 *	http://mediaportal.sourceforge.net
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

using System;
using System.Collections;
using System.Globalization;
using MediaPortal.GUI.Library;
using MediaPortal.Util;
using MediaPortal.Dialogs;
using MediaPortal.TV.Database;
using MediaPortal.TV.Recording;
using MediaPortal.Player;

namespace MediaPortal.GUI.TV
{
	/// <summary>
	/// 
	/// </summary>
	public class GUITVConflicts :GUIWindow
	{
		#region variables, ctor/dtor

		[SkinControlAttribute(10)]			protected GUIListControl listConflicts=null;
		int								m_iSelectedItem=0;
		TVRecording				currentShow=null;
		TVRecording				currentEpisode=null;
		bool							needUpdate=false;

		public  GUITVConflicts()
		{
			GetID=(int)GUIWindow.Window.WINDOW_TV_CONFLICTS;
		}
		~GUITVConflicts()
		{
		}
    
		#endregion

		#region overrides
		public override bool Init()
		{
			bool bResult=Load (GUIGraphicsContext.Skin+@"\mytvconflicts.xml");
			ConflictManager.OnConflictsUpdated+=new MediaPortal.TV.Recording.ConflictManager.OnConflictsUpdatedHandler(ConflictManager_OnConflictsUpdated);
			return bResult;
		}


		public override void OnAction(Action action)
		{
			switch (action.wID)
			{
				case Action.ActionType.ACTION_SHOW_GUI:
					if ( !g_Player.Playing && Recorder.IsViewing())
					{
						//if we're watching tv
						GUIWindowManager.ActivateWindow((int)GUIWindow.Window.WINDOW_TVFULLSCREEN);
					}
					else if (g_Player.Playing  && g_Player.IsTV && !g_Player.IsTVRecording)
					{
						//if we're watching a tv recording
						GUIWindowManager.ActivateWindow((int)GUIWindow.Window.WINDOW_TVFULLSCREEN);
					}
					else if (g_Player.Playing&&g_Player.HasVideo)
					{
						GUIWindowManager.ActivateWindow((int)GUIWindow.Window.WINDOW_FULLSCREEN_VIDEO);
					}
					break;
			}
			base.OnAction(action);
		}
		protected override void OnPageLoad()
		{
			base.OnPageLoad ();
					
			needUpdate=false;
			LoadDirectory();
					
			while (m_iSelectedItem>=GetItemCount() && m_iSelectedItem>0) m_iSelectedItem--;
			GUIControl.SelectItemControl(GetID,listConflicts.GetID,m_iSelectedItem);

		}
		protected override void OnPageDestroy(int newWindowId)
		{
			base.OnPageDestroy (newWindowId);
			m_iSelectedItem=GetSelectedItemNo();
					
			if ( !GUITVHome.IsTVWindow(newWindowId) )
			{
				if (Recorder.IsViewing() && ! (Recorder.IsTimeShifting()||Recorder.IsRecording()) )
				{
					if (GUIGraphicsContext.ShowBackground)
					{
						// stop timeshifting & viewing... 
	              
						Recorder.StopViewing();
					}
				}
			}
		}

		protected override void OnClicked(int controlId, GUIControl control, MediaPortal.GUI.Library.Action.ActionType actionType)
		{
			base.OnClicked (controlId, control, actionType);
			if (control == listConflicts)
			{
				GUIMessage msg = new GUIMessage(GUIMessage.MessageType.GUI_MSG_ITEM_SELECTED,GetID,0,control.GetID,0,0,null);
				OnMessage(msg);         
				int iItem=(int)msg.Param1;
				if (actionType == Action.ActionType.ACTION_SELECT_ITEM)
				{
					OnClick(iItem);
				}
			}

		}

		protected override void OnShowContextMenu()
		{
			OnClick(GetSelectedItemNo());
		}


		#endregion

		#region list management 
		GUIListItem GetSelectedItem()
		{
			return listConflicts.SelectedListItem;
		}

		GUIListItem GetItem(int index)
		{
			if (index < 0 || index >=listConflicts.Count) return null;
			return listConflicts[index];
		}

		int GetSelectedItemNo()
		{
			GUIMessage msg=new GUIMessage(GUIMessage.MessageType.GUI_MSG_ITEM_SELECTED,GetID,0,listConflicts.GetID,0,0,null);
			OnMessage(msg);         
			int iItem=(int)msg.Param1;
			return iItem;
		}
		int GetItemCount()
		{
			return listConflicts.Count;
		}
		#endregion

		#region scheduled tv methods
		void LoadDirectory()
		{
			GUIControl.ClearControl(GetID,listConflicts.GetID);
			GUIControl cntlLabel ;

			string strObjects;
			int total=0;
			if (currentShow!=null)
			{
				GUIListItem item= new GUIListItem();
				item.Label		  = "..";
				item.IsFolder=true;
				Utils.SetDefaultIcons(item);
				listConflicts.Add(item);
				if (currentEpisode==null && currentShow.RecType!=TVRecording.RecordingType.Once)
				{
					ArrayList showEpisode = new ArrayList();
					ConflictManager.GetConflictingSeries(currentShow, showEpisode);
					foreach (TVRecording showSerie in showEpisode)
					{
						item=new GUIListItem();
						item.Label=showSerie.Title;
						item.TVTag=showSerie;

						item.PinImage=Thumbs.TvConflictRecordingIcon;
						item.IsFolder=true;
						Utils.SetDefaultIcons(item);
						listConflicts.Add(item);
					}
					total++;
				}
				else
				{
					TVRecording eps=currentEpisode;
					if (eps==null)
						eps=currentShow;

					TVRecording[] conflicts=ConflictManager.GetConflictingRecordings(eps);

					eps.RecType=TVRecording.RecordingType.Once;
					item=new GUIListItem();
					item.Label=eps.Title;
					item.TVTag=eps;
					string strLogo=Utils.GetCoverArt(Thumbs.TVChannel,eps.Channel);
					if (!System.IO.File.Exists(strLogo))
					{
						strLogo="defaultVideoBig.png";
					}
					item.PinImage=Thumbs.TvConflictRecordingIcon;
					item.ThumbnailImage=strLogo;
					item.IconImageBig=strLogo;
					item.IconImage=strLogo;
					listConflicts.Add(item);
					total++;

					for (int i=0; i < conflicts.Length;++i)
					{
						conflicts[i].RecType=TVRecording.RecordingType.Once;
						item=new GUIListItem();
						item.Label=conflicts[i].Title;
						item.TVTag=conflicts[i];
						strLogo=Utils.GetCoverArt(Thumbs.TVChannel,conflicts[i].Channel);
						if (!System.IO.File.Exists(strLogo))
						{
							strLogo="defaultVideoBig.png";
						}
						item.PinImage=Thumbs.TvConflictRecordingIcon;
						item.ThumbnailImage=strLogo;
						item.IconImageBig=strLogo;
						item.IconImage=strLogo;
						listConflicts.Add(item);
						total++;
					}
				}
				strObjects=String.Format("{0} {1}", total, GUILocalizeStrings.Get(632));
				GUIPropertyManager.SetProperty("#itemcount",strObjects);
				cntlLabel = GetControl(12);
				if (cntlLabel!=null)
					cntlLabel.YPosition = listConflicts.SpinY;

				SetLabels();
				return;
			}
			ArrayList itemlist = new ArrayList();
			TVDatabase.GetRecordings(ref itemlist);
			foreach (TVRecording rec in itemlist)
			{
				if (!ConflictManager.IsConflict(rec)) continue;
				GUIListItem item=new GUIListItem();
				item.Label=rec.Title;
				item.TVTag=rec;
				item.PinImage=Thumbs.TvConflictRecordingIcon;
				item.IsFolder=true;
				Utils.SetDefaultIcons(item);
				listConflicts.Add(item);
				total++;
			}
      
			strObjects=String.Format("{0} {1}", total, GUILocalizeStrings.Get(632));
			GUIPropertyManager.SetProperty("#itemcount",strObjects);
			cntlLabel = GetControl(12);
			if (cntlLabel!=null)
				cntlLabel.YPosition = listConflicts.SpinY;

			SetLabels();
		}


		void SetLabels()
		{
			for (int i=0; i < GetItemCount();++i)
			{
				GUIListItem item=GetItem(i);
				if (item.IsFolder && item.Label.Equals("..")) continue;
				TVRecording rec=(TVRecording)item.TVTag;
				item.Label=rec.Title;
				string strTime=String.Format("{0} {1} - {2}", 
					rec.StartTime.ToShortDateString() , 
					rec.StartTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat),
					rec.EndTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat));
				string strType="";
				switch (rec.RecType)
				{
					case TVRecording.RecordingType.Once:
						item.Label2=String.Format("{0} {1} - {2}", 
							Utils.GetShortDayString(rec.StartTime) , 
							rec.StartTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat),
							rec.EndTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat));;
						break;
					case TVRecording.RecordingType.Daily:
						strTime=String.Format("{0}-{1}", 
							rec.StartTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat),
							rec.EndTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat));
						strType=GUILocalizeStrings.Get(648) ;
						item.Label2=String.Format("{0} {1}",strType,strTime);
						break;

					case TVRecording.RecordingType.WeekDays:
						strTime=String.Format("{0}-{1} {2}-{3}",
							GUILocalizeStrings.Get(657),//657=Mon
							GUILocalizeStrings.Get(661),//661=Fri
							rec.StartTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat),
							rec.EndTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat));
						strType=GUILocalizeStrings.Get(648) ;
						item.Label2=String.Format("{0} {1}",strType,strTime);
						break;

					case TVRecording.RecordingType.Weekly:
						string day;
						switch (rec.StartTime.DayOfWeek)
						{
							case DayOfWeek.Monday :	day = GUILocalizeStrings.Get(11);	break;
							case DayOfWeek.Tuesday :	day = GUILocalizeStrings.Get(12);	break;
							case DayOfWeek.Wednesday :	day = GUILocalizeStrings.Get(13);	break;
							case DayOfWeek.Thursday :	day = GUILocalizeStrings.Get(14);	break;
							case DayOfWeek.Friday :	day = GUILocalizeStrings.Get(15);	break;
							case DayOfWeek.Saturday :	day = GUILocalizeStrings.Get(16);	break;
							default:	day = GUILocalizeStrings.Get(17);	break;
						}

						strTime=String.Format("{0}-{1}", 
							rec.StartTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat),
							rec.EndTime.ToString("t",CultureInfo.CurrentCulture.DateTimeFormat));
						strType=GUILocalizeStrings.Get(649) ;
						item.Label2=String.Format("{0} {1} {2}",strType,day,strTime);
						break;
					case TVRecording.RecordingType.EveryTimeOnThisChannel:
						item.Label=rec.Title;
						item.Label2=GUILocalizeStrings.Get(650);
						break;
					case TVRecording.RecordingType.EveryTimeOnEveryChannel:
						item.Label=rec.Title;
						item.Label2=GUILocalizeStrings.Get(651);
						break;
				}
			}
		}

		void OnClick(int iItem)
		{
			m_iSelectedItem=GetSelectedItemNo();
			GUIListItem item = GetItem(iItem);
			if (item==null) return;
			TVRecording rec=item.TVTag as TVRecording;
			if (item.IsFolder)
			{
				if (item.Label=="..")
				{
					if (currentEpisode!=null)
						currentEpisode=null;
					else
						currentShow=null;
					LoadDirectory();
					return;
				}
				if (currentShow==null)
					currentShow=rec;
				else 
					currentEpisode=rec;
				LoadDirectory();
				return;
			}
			GUIDialogYesNo dlgYesNo = (GUIDialogYesNo)GUIWindowManager.GetWindow((int)GUIWindow.Window.WINDOW_DIALOG_YES_NO);
			if (null != dlgYesNo)
			{
				dlgYesNo.SetHeading(GUILocalizeStrings.Get(653));//Delete this recording?
				dlgYesNo.SetLine(1, rec.Channel);
				dlgYesNo.SetLine(2, rec.Title);
				dlgYesNo.SetLine(3, GUILocalizeStrings.Get(732));//are you sure
				dlgYesNo.DoModal(GUIWindowManager.ActiveWindow);

				if (dlgYesNo.IsConfirmed) 
				{
					if (rec.Series)
					{
						rec.CanceledSeries.Add(Utils.datetolong(rec.StartTime));
						TVDatabase.AddCanceledSerie(rec,Utils.datetolong(rec.StartTime));
					}
					else
					{
						TVDatabase.RemoveRecording(rec);
					}
					currentShow=null;
					LoadDirectory();
				}
			}

		}
    
		string GetRecType(TVRecording.RecordingType recType)
		{
			string strType=String.Empty;
			switch(recType)
			{
				case TVRecording.RecordingType.Daily:
					strType=GUILocalizeStrings.Get(648);//daily
					break;
				case TVRecording.RecordingType.EveryTimeOnEveryChannel:
					strType=GUILocalizeStrings.Get(651);//Everytime on any channel
					break;
				case TVRecording.RecordingType.EveryTimeOnThisChannel:
					strType=GUILocalizeStrings.Get(650);//Everytime on this channel
					break;
				case TVRecording.RecordingType.Once:
					strType=GUILocalizeStrings.Get(647);//Once
					break;
				case TVRecording.RecordingType.WeekDays:
					strType=GUILocalizeStrings.Get(680);//Mon-Fri
					break;
				case TVRecording.RecordingType.Weekly:
					strType=GUILocalizeStrings.Get(679);//Weekly
					break;
			}
			return strType;
		}
		#endregion

		
		public override bool OnMessage(GUIMessage message)
		{
			switch ( message.Message )
			{
				case GUIMessage.MessageType.GUI_MSG_ITEM_FOCUS_CHANGED:
					UpdateDescription();
					break;
			}
			return base.OnMessage(message);
		}
		void UpdateDescription()
		{
			TVRecording rec = new TVRecording();
			rec.SetProperties(null);
			GUIListItem pItem=GetItem( GetSelectedItemNo() );
			if (pItem==null)
			{
				return;
			}
			rec=pItem.TVTag as TVRecording;
			if (rec==null) return;
			TVProgram prog=ConflictManager.Util.GetProgramAt(rec.Channel,rec.StartTime.AddMinutes(1));
			rec.SetProperties(prog);
		}

		private void ConflictManager_OnConflictsUpdated()
		{
			needUpdate=true;
		}
		public override void Process()
		{
			if (needUpdate)
			{
				needUpdate=false;
				LoadDirectory();
			}
		}
	}
}
